@@ -25,6 +25,7 @@
 
 #include <sys/socket.h>
 #include <sys/wait.h>
+#include <sys/stat.h>
 #include <fcntl.h>
 #include <signal.h>
 #include <time.h>
@@ -350,15 +351,23 @@ int prepare_buffer(pipe_t* pipe)
 		/* in line-based mode an intermediate ring-buffer is used for
 		 * faster data processing */
 		if (!pipe->charbased) {
-			len = pipe->rblen + 1;
-			if (!(pipe->rdbuf.start = (char*)malloc(len))) {
+			struct stat statbuf;
+			/* get pipe stats to determine optimal buffer size*/
+			if (fstat(pipe->fh, &statbuf)) {
+				error("unable to get io buffer stats for %s\n",
+				      pipe->name);
+				return -1;
+			}
+			pipe->rdbuf.len = statbuf.st_blksize;
+
+			pipe->rdbuf.start = (char*)malloc(pipe->rdbuf.len);
+			if (!pipe->rdbuf.start) {
 				error_outofmemory();
 				error("trying to allocate %d "
-				      "byte %s  read buffer\n",
-				      pipe->rblen + 1, pipe->name);
+				      "byte %s read buffer\n",
+				      pipe->rdbuf.len, pipe->name);
 				return -1;
 			}
-			pipe->rdbuf.len = pipe->rblen;
 			pipe->rdbuf.read = pipe->rdbuf.start;
 			pipe->rdbuf.end = pipe->rdbuf.start;
 		}
@@ -499,7 +508,16 @@ int handle_lbstream(pipe_t* pipe)
 		if (pipe->execregexp) {
 			if (!regexec(&pipe->pexecreg, pipe->buf, pipe->bfill,
 				     NULL, 0)) {
-				system(pipe->execcommand);
+				int ret = system(pipe->execcommand);
+				if (ret) {
+					print(config.exitonexecfail
+					      ?ERROR:WARNING,
+					      "command \"%s\" returned 0x%x\n",
+					      pipe->execcommand, ret);
+					if (config.exitonexecfail) {
+						exit(EXIT_FAILURE);
+					}
+				}
 			}
 		}
 	} else {
