@@ -68,6 +68,8 @@ config_t config = {
 	.custconfigfile		= NULL,
 	.preexeccmd		= NULL,
 	.postexeccmd		= NULL,
+	.exitonexecfail		= 0,
+	.extregexp		= 0,
 };
 
 /* default search path list for configuration file */
@@ -89,7 +91,6 @@ pipe_t pstdout = {
 	.switchtocbmode = 0,
 	.blen		= 65540,
 	.bfill		= 0,
-	.rblen		= 4096,
 	.memcr		= 0,
 	.recalclen	= 0,
 	.fgcol		= -1,
@@ -120,7 +121,6 @@ pipe_t pstderr = {
 	.switchtocbmode = 0,
 	.blen		= 65540,
 	.bfill		= 0,
-	.rblen		= 4096,
 	.memcr		= 0,
 	.recalclen	= 0,
 	.fgcol		= 1,
@@ -255,6 +255,9 @@ arglist_t arglist[] = {
 		"handle stdout char- instead of line-based", 0 },
 	{ '\0', "stderr_charbased",	"BOOL",	TBOOL,  &pstderr.charbased,
 		"handle stderr char- instead of line-based", 0 },
+	{ '\0', "extended-regexp",	"BOOL", TBOOL,	&config.extregexp,
+		"interpret regexp patterns as extended regular expressions",
+		0 },
 	/* Logging options */
 	{ '\0', NULL,		NULL,		TNONE,	NULL,
 		"Logging options:", 0 },
@@ -329,6 +332,8 @@ arglist_t arglist[] = {
 	/* Command execution options */
 	{ '\0', NULL,		NULL,		TNONE,	NULL,
 		"Command execution options:", 0 },
+	{ '\0', "exitonexecfail", "BOOL",	TBOOL,	&config.exitonexecfail,
+		"exit if execution of command fails", 0 },
 	{ '\0', "preexec",	"STRING",	TSTRING, &config.preexeccmd,
 		"command executed before application start", 0 },
 	{ '\0', "postexec",	"STRING",	TSTRING, &config.postexeccmd,
@@ -379,6 +384,7 @@ void show_configuration(void)
 	message("  rename logfiles         %i\n", config.logrename);
 	message("  circular logfile        %i\n", config.circularlog);
 	message("  extension for old logs  %s\n", logfile.oldext);
+	message("  exit on execution fail  %i\n", config.exitonexecfail);
 	
 	message("\n  application executable  %s\n",
 		app.exe?app.exe:"<none>");
@@ -408,6 +414,8 @@ void show_configuration(void)
 		message("  remove CR for PTY       <not supported>\n");
 	}
 
+	message("\n  ext. regexp patterns    %i\n", config.extregexp);
+
 	message("\n  dumb terminal mode      %i\n", config.dumbterm);
 	message("  print_summary           %i\n", config.printsummary);
 	message("  print_logname           %i\n", config.printlogname);
@@ -584,9 +592,14 @@ static int fixup_pipe(pipe_t *pipe)
 		pipe->esccolorlen += 4;
 	}
 
+	int regexp_cflags = REG_NOSUB|REG_NEWLINE;
+	if (config.extregexp)
+		regexp_cflags |= REG_EXTENDED;
+
 	if (pipe->regexp) {
 		if (strlen(pipe->regexp)) {
-			if (regcomp(&pipe->preg, pipe->regexp, REG_NOSUB)) {
+			if (regcomp(&pipe->preg, pipe->regexp,
+				    regexp_cflags)) {
 				error("unable to process regular expression "
 				      "for %s\n", pipe->name);
 				return -1;
@@ -599,7 +612,7 @@ static int fixup_pipe(pipe_t *pipe)
 	if (pipe->execregexp) {
 		if (strlen(pipe->execregexp)) {
 			if (regcomp(&pipe->pexecreg, pipe->execregexp,
-				    REG_NOSUB)) {
+				    regexp_cflags)) {
 				error("unable to process regular expression "
 				      "for %s\n", pipe->name);
 				return -1;
@@ -821,20 +834,27 @@ int get_argvalue(int priority, int argid, int* argp, int argc, char* argv[],
 		return -1;
 	}
 
-	if (value && (arglist[argid].type == TNONE)) {
+	if (value == &argv[*argp][2]) {
+		/* Long parameter has no value provided - we assume 'true' for
+		 * boolean options */
+		if (arglist[argid].type == TBOOL) {
+			value = boolvalues[0].string;
+		}
+	} else if (value && (arglist[argid].type == TNONE)) {
 		error("no value expected\n");
 		return -1;
 	} else if (!value && (arglist[argid].type != TNONE))  {
 		if (*argp < argc -1) {
 			(*argp)++;
-			if (argv[*argp][0]!='-')
+			if (argv[*argp][0]!='-') {
 				value = argv[*argp];
+			}
 		}
 		if (!value) {
 			error("parameter needs value\n");
 			return -1;
 		}
-	}
+	} 
 
 	switch (arglist[argid].type) {
 		case TNONE:	if (arglist[argid].set > priority)
@@ -951,6 +971,10 @@ int parse_args(int argc, char* argv[])
 					if (*(value+1)) {
 						value++;
 					} 
+				} else {
+					/* mark a long option without provided
+					 * value */
+					value = &argv[i][2];
 				}
 
 				argid = get_argid(arg, 0);
