@@ -2,7 +2,7 @@
  * logapp.h: Logapp main header file
  *
  *
- * Copyright (C) 2007-2008 Michael Brunner <mibru@gmx.de>
+ * Copyright (C) 2007-2009 Michael Brunner <mibru@gmx.de>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -19,16 +19,17 @@
 #ifndef __LOGAPP_H__
 #define __LOGAPP_H__
 
-#define VERSION		"0.10"
+#define VERSION		"0.12"
 #define EXECUTABLE	"logapp"
 #define AUTHOR		"Michael Brunner <mibru@gmx.de>"
 
-#define message(x...)	print(0, x)
-#define warning(x...)	print(1, x)
-#define error(x...)	print(2, x)
+typedef enum { MESSAGE, WARNING, ERROR } t_printtype;
+#define message(x...)	print(MESSAGE, x)
+#define warning(x...)	print(WARNING, x)
+#define error(x...)	print(ERROR, x)
 #define STRING_WARNING	" warning: "
 #define STRING_ERROR	" error: "
-void print(int type, const char *format,...);
+void print(t_printtype type, const char *format,...);
 void error_outofmemory(void);
 
 #define CONFIG_SUPPORT_PTY	1
