@@ -50,6 +50,8 @@ extern char **environ;
 static int logfile_add_header(logfile_t* logfile)
 {
 	int ret;
+	int len;
+	int headlen;
 	int i;
 	struct tm* loctime;
 	char buf[MAX_HEADER + 1];
@@ -58,8 +60,7 @@ static int logfile_add_header(logfile_t* logfile)
 
 	/* Get to the end of the logfile and check size */
 	if (lseek(logfile->fh, 0, SEEK_END) != 0) {
-		strcpy(buf, "\n");
-		if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+		if (write(logfile->fh, "\n", 1) == -1)
 			return -1;
 
 		logfile_check_size(logfile, 0, MAX_HEADER * 2);
@@ -67,21 +68,27 @@ static int logfile_add_header(logfile_t* logfile)
 
 	buf[MAX_HEADER] = '\0';
 
+	headlen = strlen(logfile->head);
+
 	/* Write application name */
-	snprintf(buf, MAX_HEADER, "%s %s\n", logfile->head, app.exe);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	len = snprintf(buf, MAX_HEADER, "%s %s\n", logfile->head, app.exe);
+	if ((len < 0) || (len > MAX_HEADER)) {
+		return -1;
+	}
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 
 	/* Write argument list */
 	for (i=1; i<app.argc; i++) {
-		if (write(logfile->fh, logfile->head, strlen(logfile->head))
-		    != strlen(logfile->head))
+		if (write(logfile->fh, logfile->head, headlen) == -1)
 			return -1;
-		snprintf(buf, MAX_HEADER, " arg[%d] = \"", i);
-		if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+		len = snprintf(buf, MAX_HEADER, " arg[%d] = \"", i);
+		if ((len < 0) || (len > MAX_HEADER)) {
 			return -1;
-		if (write(logfile->fh, app.argv[i], strlen(app.argv[i]))
-		    != strlen(app.argv[i]))
+		}
+		if (write(logfile->fh, buf, len) == -1)
+			return -1;
+		if (write(logfile->fh, app.argv[i], strlen(app.argv[i])) == -1)
 			return -1;
 		if (write(logfile->fh, "\"", 1) != 1)
 			return -1;
@@ -90,8 +97,7 @@ static int logfile_add_header(logfile_t* logfile)
 				return -1;
 	}
 	if (app.argc<2) {
-		if (write(logfile->fh, logfile->head, strlen(logfile->head))
-		    != strlen(logfile->head))
+		if (write(logfile->fh, logfile->head, headlen) == -1)
 			return -1;
 		if (write(logfile->fh, " <no command line options>", 26)
 		    != 26 )
@@ -101,8 +107,11 @@ static int logfile_add_header(logfile_t* logfile)
 		return -1;
 
 	/* Write current path */
-	snprintf(buf, MAX_HEADER, "%s pwd = ", logfile->head);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	len = snprintf(buf, MAX_HEADER, "%s pwd = ", logfile->head);
+	if ((len < 0) || (len > MAX_HEADER)) {
+		return -1;
+	}
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 	if (getcwd(pwd, MAX_PWD) == NULL) {
 		if (write(logfile->fh, "<unable to get working directory>\n",
@@ -111,23 +120,23 @@ static int logfile_add_header(logfile_t* logfile)
 	} else {
 		pwd[strlen(pwd)+1] = '\0';
 		pwd[strlen(pwd)] = '\n';
-		if (write(logfile->fh, pwd, strlen(pwd)) != strlen(pwd))
+		if (write(logfile->fh, pwd, strlen(pwd)) == -1)
 			return -1;
 	}
 
 	/* Write environment variables */
 	if (config.logenv) {
 		for (i=0; environ[i] != NULL; i++) {
-			if (write(logfile->fh, logfile->head,
-				  strlen(logfile->head))
-			    != strlen(logfile->head))
+			if (write(logfile->fh, logfile->head, headlen) == -1)
 				return -1;
-			snprintf(buf, MAX_HEADER, " env[%d] = \"", i);
-			if (write(logfile->fh, buf, strlen(buf))
-			    != strlen(buf))
+			len = snprintf(buf, MAX_HEADER, " env[%d] = \"", i);
+			if ((len < 0) || (len > MAX_HEADER)) {
+				return -1;
+			}
+			if (write(logfile->fh, buf, len) == -1)
 				return -1;
 			if (write(logfile->fh, environ[i], strlen(environ[i]))
-			    != strlen(environ[i]))
+			    == -1)
 				return -1;
 			if (write(logfile->fh, "\"\n", 2) != 2)
 				return -1;
@@ -142,7 +151,10 @@ static int logfile_add_header(logfile_t* logfile)
 		tmp[0] = '\0';
 	}
 	snprintf(buf, MAX_HEADER, "%s %s\n", logfile->head, tmp);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	if ((len < 0) || (len > MAX_HEADER)) {
+		return -1;
+	}
+	if (write(logfile->fh, buf, strlen(buf)) == -1)
 		return -1;
 
 	return 0;
@@ -151,6 +163,7 @@ static int logfile_add_header(logfile_t* logfile)
 static int logfile_add_splitmarker(logfile_t* logfile)
 {
 	int ret;
+	int len;
 	time_t sec;
 	struct tm* loctime;
 	char buf[MAX_SPLIT + 1];
@@ -159,8 +172,11 @@ static int logfile_add_splitmarker(logfile_t* logfile)
 	buf[MAX_SPLIT] = '\0';
 
 	/* Write application name */
-	snprintf(buf, MAX_SPLIT, "%s %s\n", logfile->split, app.exe);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	len = snprintf(buf, MAX_SPLIT, "%s %s\n", logfile->split, app.exe);
+	if ((len < 0) || (len > MAX_SPLIT)) {
+		return -1;
+	}
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 
 	/* Write starttime */
@@ -170,16 +186,23 @@ static int logfile_add_splitmarker(logfile_t* logfile)
 	if ((app.starttime < 0) || (ret == 0)) {
 		tmp[0] = '\0';
 	}
-	snprintf(buf, MAX_SPLIT, "%s running since %s\n", logfile->split, tmp);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	len = snprintf(buf, MAX_SPLIT, "%s running since %s\n", logfile->split,
+		       tmp);
+	if ((len < 0) || (len > MAX_SPLIT)) {
+		return -1;
+	}
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 
 	/* Write current line count */
-	snprintf(buf, MAX_SPLIT, "%s %d lines output (%d stdout, %d stderr)"
-		 "\n", logfile->split,
+	len = snprintf(buf, MAX_SPLIT, "%s %d lines output (%d stdout, %d"
+		       " stderr)\n", logfile->split,
 		 app.pstdout-> linecount + app.pstderr->linecount,
 		 app.pstdout->linecount, app.pstderr->linecount);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	if ((len < 0) || (len > MAX_SPLIT)) {
+		return -1;
+	}
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 
 	/* Write execution time */
@@ -193,9 +216,12 @@ static int logfile_add_splitmarker(logfile_t* logfile)
 	}
 
 	if ((app.starttime > -1)&&(app.starttime <= sec)) {
-		snprintf(buf, MAX_SPLIT, "%s execution time is %us\n",
+		len = snprintf(buf, MAX_SPLIT, "%s execution time is %us\n",
 			 logfile->split, (unsigned int)(sec - app.starttime));
-		if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+		if ((len < 0) || (len > MAX_SPLIT)) {
+			return -1;
+		}
+		if (write(logfile->fh, buf, len) == -1)
 			return -1;
 	} else {
 		error("problem calculating execution time\n");
@@ -206,7 +232,8 @@ static int logfile_add_splitmarker(logfile_t* logfile)
 
 static int logfile_add_footer(logfile_t* logfile)
 {
-	int ret = 0;
+	int len = 0;
+	int tmplen = 0;
 	time_t sec;
 	struct tm* loctime;
 	char buf[MAX_FOOTER + 1];
@@ -221,56 +248,76 @@ static int logfile_add_footer(logfile_t* logfile)
 	/* Write exit time */
 	if (time(&sec) > -1) {
 		loctime = localtime(&sec);
-		ret = strftime(tmp, MAX_TMP, "%Y-%m-%d %T %Z", loctime);
+		len = strftime(tmp, MAX_TMP, "%Y-%m-%d %T %Z", loctime);
 	} 
 	
-	if ((sec < 0) || (ret == 0)) {
+	if ((sec < 0) || (len < 1)) {
 		tmp[0] = '\0';
+		len = 0;
 	}
 
 	snprintf(buf, MAX_FOOTER, "%s %s\n", logfile->foot, tmp);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	if ((len < 0) || (len > MAX_FOOTER)) {
+		return -1;
+	}
+	len += strlen(logfile->foot) + 2;
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 
 	/* Write exit status */
-	snprintf(buf, MAX_FOOTER, "%s application exit status: ",
+	len = snprintf(buf, MAX_FOOTER, "%s application exit status: ",
 		 logfile->foot);
+	if ((len < 0) || (len > MAX_FOOTER)) {
+		return -1;
+	}
 	if (WIFEXITED(app.exit_state)) {
-		snprintf(buf + strlen(buf), MAX_FOOTER - strlen(buf), "%d\n",
+		tmplen = snprintf(buf + len, MAX_FOOTER - len, "%d\n",
 			 WEXITSTATUS(app.exit_state));
 	} else if (WIFSIGNALED(app.exit_state)) {
-		snprintf(buf + strlen(buf), MAX_FOOTER - strlen(buf),
+		tmplen = snprintf(buf + len, MAX_FOOTER - len,
 			 "abnormal termination (signal = %d)\n",
 			 WTERMSIG(app.exit_state));
 	} else {
-		snprintf(buf + strlen(buf), MAX_FOOTER - strlen(buf),
-			 "unknown\n");
+		tmplen += snprintf(buf + len, MAX_FOOTER - len, "unknown\n");
 	}
+	if ((tmplen < 0) || (tmplen > MAX_FOOTER)) {
+		return -1;
+	}
+	len += tmplen;
 
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 
 	/* Write line counts */
-	snprintf(buf, MAX_FOOTER, "%s %d lines output (%d stdout, %d stderr)"
-		 "\n", logfile->foot,
+	len = snprintf(buf, MAX_FOOTER, "%s %d lines output (%d stdout, %d"
+		       " stderr)\n", logfile->foot,
 		 app.pstdout-> linecount + app.pstderr->linecount,
 		 app.pstdout->linecount, app.pstderr->linecount);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	if ((len < 0) || (len > MAX_FOOTER)) {
+		return -1;
+	}
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 
 	/* Write execution time */
 	if ((app.starttime > -1)&&(app.starttime <= sec)) {
-		snprintf(buf, MAX_FOOTER, "%s execution time was %us\n",
+		len = snprintf(buf, MAX_FOOTER, "%s execution time was %us\n",
 			 logfile->foot, (unsigned int)(sec - app.starttime));
-		if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+		if ((len < 0) || (len > MAX_FOOTER)) {
+			return -1;
+		}
+		if (write(logfile->fh, buf, len) == -1)
 			return -1;
 	} else {
 		error("problem calculating execution time\n");
 	}
 
 	/* Write application name */
-	snprintf(buf, MAX_FOOTER, "%s %s\n", logfile->foot, app.exe);
-	if (write(logfile->fh, buf, strlen(buf)) != strlen(buf))
+	len = snprintf(buf, MAX_FOOTER, "%s %s\n", logfile->foot, app.exe);
+	if ((len < 0) || (len > MAX_FOOTER)) {
+		return -1;
+	}
+	if (write(logfile->fh, buf, len) == -1)
 		return -1;
 
 	return 0;
@@ -279,6 +326,7 @@ static int logfile_add_footer(logfile_t* logfile)
 static int logfile_rename(logfile_t* logfile, char* oldname)
 {
 	char* newname;
+	int ret = 0;
 
 	if (!config.logrename)
 		return 0;
@@ -289,7 +337,7 @@ static int logfile_rename(logfile_t* logfile, char* oldname)
 	newname = malloc(strlen(oldname) 
 			 + sizeof(logfile->oldext));
 	if (newname == NULL) {
-		error("out of memory during logfile rename\n");
+		error_outofmemory();
 		return -1;
 	}
 
@@ -298,30 +346,34 @@ static int logfile_rename(logfile_t* logfile, char* oldname)
 
 	if (rename(oldname, newname)) {
 		if (errno != ENOENT) {
-			free(newname);
-			error("logfile rename failed (error: %d)\n", errno);
-			return -1;
+			ret = -1;
+			warning("logfile rename failed (error: %d)\n", errno);
 		}
 	}
 
 	free(newname);
 
-	return 0;
+	return ret;
 }
 
 static char* logfile_get_alternate_name(logfile_t* logfile)
 {
+	int len;
+
 	if (!logfile->appendnr) {
 		return strdup(logfile->name);
 	}
 
 	char number[12];
 
-	snprintf(number, 12, ".%d", logfile->appendnr);
+	len = snprintf(number, 12, ".%d", logfile->appendnr);
+	if (len < 0) {
+		return NULL;
+	}
 	
 	char *name = malloc(strlen(logfile->name) + strlen(number) + 1);
 	if (name == NULL) {
-		error("out of memory during logfile split\n");
+		error_outofmemory();
 		return NULL;
 	}
 
@@ -374,12 +426,13 @@ int logfile_write(logfile_t* logfile, pipe_t* pipe)
 	unsigned stamp_sec = 0;
 	int stamp_msec = 0;
 	int writetimestamp = config.logtime;
+	int prefixlen = 0;
 
 	if (!logfile->name)
 		return 0;
 
 	if (CONFIG_USE_THREADS)
-		pthread_mutex_lock( &logfile_mutex);
+		pthread_mutex_lock(&logfile_mutex);
 
 	if (logfile->sizelimit)
 		logfile_check_size(logfile, 1, pipe->bfill);
@@ -461,43 +514,90 @@ int logfile_write(logfile_t* logfile, pipe_t* pipe)
 	/* Write timestamp */
 	if (writetimestamp) {
 		char timestamp[16];
+		int len;
 
 		/* Normally use a 9 digit timestamp but extend the size to 13
 		 * digits if needed */
 		if (stamp_sec < 999999) {
-			snprintf(timestamp, 12, "%06u%03u: ", stamp_sec,
+			len = snprintf(timestamp, 12, "%06u%03u: ", stamp_sec,
 				 stamp_msec);
-			write(logfile->fh, timestamp, 11);
+			if (len < 0)
+				goto failure_exit;
+			prefixlen = 11;
 		} else {
-			snprintf(timestamp, 16, "%010u%03u: ", stamp_sec,
+			len = snprintf(timestamp, 16, "%010u%03u: ", stamp_sec,
 				 stamp_msec);
-			write(logfile->fh, timestamp, 15);
+			if (len < 0)
+				goto failure_exit;
+			prefixlen = 15;
 		}
-
+		if (write(logfile->fh, timestamp, prefixlen) == -1)
+			goto failure_exit;
 	}
 
 	/* Write prefix */
 	if (pipe->lineprefix != NULL) {
-		if (write(logfile->fh, pipe->lineprefix,
-			  strlen(pipe->lineprefix))
-		    != strlen(pipe->lineprefix)) {
-			if (CONFIG_USE_THREADS)
-				pthread_mutex_unlock( &logfile_mutex);
-			return -1;
+		if (write(logfile->fh, pipe->lineprefix, pipe->lineprefixlen)
+		    == -1) {
+			goto failure_exit;
+		}
+		prefixlen += pipe->lineprefixlen;
+	}
+
+	int i = 0;
+	int offset = 0;
+
+	if (config.alignlinebreaks && pipe->charbased) {
+		while(pipe->linebreakpos[i]) {
+			if (i) {
+				if (write(logfile->fh, logfile->indent,
+					  prefixlen) == -1) {
+					goto failure_exit;
+				}
+			}
+			if (write(logfile->fh, pipe->buf+offset,
+				  pipe->linebreakpos[i]-offset) == -1) {
+				goto failure_exit;
+			}
+			offset = pipe->linebreakpos[i];
+			i++;
+		}
+		if (!(CONFIG_SUPPORT_PTY && config.ptyremovecr
+		      && config.usepty) || (i>=LINEBREAKBUFSIZE)) {
+			for (i=offset; i<pipe->bfill; i++) {
+				if (pipe->buf[i]=='\n' || (i+1==pipe->bfill)) {
+					if (offset) {
+						if (write(logfile->fh,
+							  logfile->indent,
+							  prefixlen) == -1) {
+							goto failure_exit;
+						}
+					}
+					if (write(logfile->fh, pipe->buf + 
+						  offset, i-offset + 1)
+					    == -1) {
+						goto failure_exit;
+					}
+					offset = i+1;
+				}
+			}
+		}
+	} else {
+		/* Write data */
+		if (write(logfile->fh, pipe->buf, pipe->bfill) == -1) {
+			goto failure_exit;
 		}
 	}
 
-	/* Write data */
-	if (write(logfile->fh, pipe->buf, pipe->bfill) != pipe->bfill) {
-		if (CONFIG_USE_THREADS)
-			pthread_mutex_unlock( &logfile_mutex);
-		return -1;
-	}
-
 	if (CONFIG_USE_THREADS)
-		pthread_mutex_unlock( &logfile_mutex);
+		pthread_mutex_unlock(&logfile_mutex);
 
 	return 0;
+
+failure_exit:
+	if (CONFIG_USE_THREADS)
+		pthread_mutex_unlock(&logfile_mutex);
+	return -1;
 }
 
 int get_lock(logfile_t* logfile)
@@ -543,8 +643,14 @@ int logfile_open(char* filename, logfile_t* logfile)
 			logfile->fh = open(altname, O_WRONLY, 0644);
 			if (logfile->fh != -1) {
 				if (!get_lock(logfile)) {
-					logfile_rename(logfile, altname);
 					close(logfile->fh);
+					if (logfile_rename(logfile, altname)) {
+						/* unable to rename logfile
+						   -> try next slot */
+						logfile->fh = -1;
+						logfile->appendnr++;
+						continue;
+					}
 				} else {
 					/* it is locked, so try next slot */
 					close(logfile->fh);
